<?php
/**
 * Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2019 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * http://cloudlinux.com/docs/LICENSE.TXT
 */


use Modules_PleskLvemanager_CheckData as CheckData;
use Modules_PleskLvemanager_Controllers_Spa as Spa;

class IndexController extends Modules_PleskLvemanager_Controllers_Admin
{
    const DEFAULT_LANGUAGE = 'en-US';

    protected $_accessLevel = array('admin', 'reseller');

    public function indexAction()
    {
        if ($_SERVER['REQUEST_METHOD'] != 'GET') {
            $this->sendErrorResponse('Method Not Allowed', $statusCode = 405);
        }

        if ($this->client->isAdmin()) {
            $userType = Spa::OWNER_ADMIN;
        } else if ($this->client->isReseller()) {
            $userType = Spa::OWNER_RESELLER;
        } else {
            $userType = Spa::OWNER_USER;
        }

        // Rendering of main template.
        if (!isset($_COOKIE['csrftoken'])) {
            setcookie("csrftoken", md5(uniqid(rand(), true)), time() + 86400, '/', '', true);
        }

        $this->view->pleskVersion = str_replace(
            '.', '',
            number_format((float)CheckData::getPleskVersion(), 1)
        );

        $this->view->pluginVersion = CheckData::getPluginVersion();
        $this->view->owner = $userType;
        $this->view->userName = $this->getLogin();
        $this->view->appMode = CheckData::getAppMode();
        $currentLanguage = pm_Locale::getCode();
        $this->view->currentLanguage = $currentLanguage;
    }
}
