<?php
/**
 * Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2019 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * http://cloudlinux.com/docs/LICENSE.TXT
 */


use Modules_PleskLvemanager_CheckData as CheckData;

class Modules_PleskLvemanager_CustomButtons extends pm_Hook_CustomButtons
{
    public function getButtons()
    {
        $buttons = array();
        $xrayButton = array(
                 'place' => self::PLACE_DOMAIN,
                 'title' => pm_Locale::lmsg('X-Ray'),
                 'description' => pm_Locale::lmsg('X-Ray'),
                 'icon' => pm_Context::getBaseUrl() . 'images/xray.svg',
                 'link' => pm_Context::getActionUrl('xray'));
        $phpSelectorButton = array(
                 'place' => self::PLACE_DOMAIN,
                 'title' => pm_Locale::lmsg('PHP Selector'),
                 'description' => pm_Locale::lmsg('PHP Selector'),
                 'icon' => pm_Context::getBaseUrl() . 'images/php_selector_logo.svg',
                 'link' => pm_Context::getActionUrl('php-selector'));
        $resourceUsageButton = array(
                 'place' => self::PLACE_DOMAIN,
                 'title' => pm_Locale::lmsg('Resource Usage'),
                 'description' => pm_Locale::lmsg('Resource Usage Details'),
                 'icon' => pm_Context::getBaseUrl() . 'images/resource_usage_logo.png',
                 'link' => pm_Context::getActionUrl('resource-usage'));
        $awpButton = array(
                 'place' => self::PLACE_DOMAIN,
                 'title' => pm_Locale::lmsg('AccelerateWP'),
                 'description' => pm_Locale::lmsg('AccelerateWP'),
                 'icon' => pm_Context::getBaseUrl() . 'images/awp.svg',
                 'link' => pm_Context::getActionUrl('awp'));

        if (version_compare(pm_ProductInfo::getVersion(), '12.0') >= 0) {
            $buttons = array();

            $hideXrayApp = CheckData::getLveParam('hideXrayApp', true);  // possible values: null|true|false
            $hideAwpApp = CheckData::getLveParam('hideAccelerateWPApp', true);   // possible values: null|true|false
            if (CheckData::isClSoloEdition() || (is_bool($hideXrayApp) && !$hideXrayApp)) {
                array_push($buttons, $xrayButton);
            }
            if (CheckData::isClSoloEdition() || (is_bool($hideAwpApp) && !$hideAwpApp)) {
                array_push($buttons, $awpButton);
            }
            if (!CheckData::getLveParam('hidePhpApp', false) && !CheckData::isClSoloEdition()) { // WARN: same default exists in cloudlinux-selector code
                array_push($buttons, $phpSelectorButton);
            }
            if (!CheckData::getLveParam('hideLVEUserStat', false) && !CheckData::isClSoloEdition()) { // By default resource usage is enabled (second param)
                array_push($buttons, $resourceUsageButton);
            }

            array_push($buttons, array(
                'place' => array_filter([
                    self::PLACE_RESELLER_TOOLS_AND_SETTINGS,
                    defined('self::PLACE_RESELLER_NAVIGATION') ? self::PLACE_RESELLER_NAVIGATION : false,
                    defined('self::PLACE_ADMIN_NAVIGATION') ? self::PLACE_ADMIN_NAVIGATION : false
                ]),
                'title' => pm_Locale::lmsg('CloudLinux Manager'),
                'description' => pm_Locale::lmsg('CloudLinux Manager'),
                'icon' => pm_Context::getBaseUrl() . 'images/addon_CloudLinux_logo.png',
                'link' => pm_Context::getActionUrl('index'),
            ));
        }
        return $buttons;
    }
}
