<?php
/**
 * Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2019 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * http://cloudlinux.com/docs/LICENSE.TXT
 */

use Modules_PleskLvemanager_Controllers_Spa as Spa;

class Modules_PleskLvemanager_CheckData
{
    // Default app mode
    const APP_MODE = 'PRODUCTION_MODE';

    public static function getLogin($session, $client, $clientId, $userType, $dropReseller = false)
    {
        $login = '';

        $db = new Modules_PleskLvemanager_DataSource();

        if ($client->isReseller() || $client->isAdmin()) {
            $clientId = isset($_SESSION['auth']['smbImpersonatedClientId'])
                ?  $_SESSION['auth']['smbImpersonatedClientId'] : $client->getId();
            $login = $client->getProperty('login');
        }
        if (isset($clientId) && $userType == Spa::OWNER_USER) {
            if (version_compare(self::getPleskVersion(), '12.0') >= 0) {
                $domain = $session->getCurrentDomain();
                $domainId = $domain->getId();
            } else {
                $domainId = $_SESSION['subscriptionId']->current;
            }

            if (isset($domainId)) {
                $logins = $db->get_user_by_id_and_domainId($clientId, $domainId);
                if (count($logins) == 1) {
                    $login = $logins[0];
                } else {
                    $ftp_user = self::getSysUser();

                    if (in_array($ftp_user, $logins)) {
                        $login = $ftp_user;
                    }
                }
            }
        }
        if ($login === '') {
            $login = self::getSysUser();
            $clientId = $db->get_id_by_impersonated_user($login);
            $impersonated_id = isset($_SESSION['auth']['smbImpersonatedClientId'])
                ?  $_SESSION['auth']['smbImpersonatedClientId'] : $client->getId();
            if ( $clientId != $impersonated_id ) {
                throw new pm_Exception("You are not authorized to view this Resource", 1);
            }
        }

        $db->endSession();

        if (!strlen($login)) {
            throw new pm_Exception("Cannot find user ID. Please ensure that domain has subscription and system user.", 1);
        }

        if ($dropReseller && $client->isReseller()) {
            throw new pm_Exception("Resellers are not authorized to view this Resource.", 1);
        }

        return $login;
    }

    public static function getUID($login)
    {
        $uid = exec('/usr/bin/getent passwd ' . trim($login) . ' | cut -d: -f3');
        if ($uid == '') {
            die('Cannot find user ID. Please ensure that domain has subscription and system user.');
        }
        return $uid;

    }

    public static function getSysUser()
    {
        $system_user = '';
        if (isset($_GET['ftp_user'])) {
            $system_user_not_valid = $_GET['ftp_user'];
        } elseif (isset($_SESSION['system_user'])) {
            $system_user_not_valid = $_SESSION['system_user'];
        }

        if (isset($system_user_not_valid)) {
            if (preg_match("/([\w\-_.]+)/", $system_user_not_valid, $captured)) {
                $system_user = $captured[1];
                if (empty($_SESSION['system_user'])) {
                    $_SESSION['system_user'] = $system_user;
                }
            }
        }

        return $system_user;
    }

    public static function getPleskVersion()
    {
        $version = explode(' ', exec('cat /usr/local/psa/version'));
        return $version[0];
    }

    public static function getPluginVersion()
    {
        return chop(file_get_contents('/usr/share/l.v.e-manager/version'));
    }

    public static function getAppMode()
    {
        $modeFile = '/usr/share/l.v.e-manager/spa/app_mode.status';

        if (file_exists($modeFile)) {
            return trim(file_get_contents($modeFile));
        }

        return self::APP_MODE;
    }

    public static function getLveParam($param, $defaultContent = null)
    {
        $uiConfigFile = '/usr/share/l.v.e-manager/lvemanager-config.json';
        $content = $defaultContent;

        if (file_exists($uiConfigFile)) {
            $content = file_get_contents($uiConfigFile, true);
            try {
                $parsedJson = json_decode($content, JSON_INVALID_UTF8_IGNORE)['ui_config']['uiSettings'];
                $content = isset($parsedJson[$param]) ? $parsedJson[$param] : null;
            } catch (Exception $e) {
                // Default content will be returned
            }
        }

        return $content;
    }

    public static function isClSoloEdition()
    {
        $solo_edition_marker = '/etc/cloudlinux-edition-solo';
        if (file_exists($solo_edition_marker)) {
            return true;
        }
        return false;
    }
}
